import java.util.ArrayList;

import rl.AbstractReinforcementLearning;
import rl.simplerl.SimpleReinforcementLearning;
import rule.AirSlide;
import rule.DistanceAirFireball;
import rule.DistanceFireball;
import rule.EmptyRule;
import rule.FireballCounter;
import rule.FleeJump;
import rule.KneeAirCounter;
import rule.KneeAirCounterHighPrio;
import rule.MegaFireballCounter;
import rule.MegaFireballEnemyDownHighPrio;
import rule.MegaFireballTimeExtremeLow;
import rule.MegaFireballTimeLow;
import rule.NearAirFireball;
import rule.PermanentDiagonalFireball;
import rule.PermanentHardAirPunch;
import rule.PermanentHardAirPunch2;
import rule.PermanentHardAirPunch2HighPrio;
import rule.PermanentSlideCounter;
import rule.PermanentSlideDefault;
import rule.PermanentSlideHighPrio;
import rule.Rule;
import rule.ShortAttackCounter;
import rule.ShortSlideCounter;
import rule.SlideCounter;
import rule.SlideSlideCombo;
import rule.Uppercut;
import rule.UppercutAirCounter;
import rule.UppercutAirCounterHighPrio;
import rule.WrongDirectionCounter;

/**
 * Code Monkey - Adaptive Dynamic Scripting AI. Version 1.1
 * 
 * @author Kevin Majchrzak
 * 
 */
public class CodeMonkey extends DynamicScriptingAI {

	private static final int DS_SIZE = 20;
	private static final int MAX_EMPTY_RULE_COUNT = 19;
	AbstractReinforcementLearning rl;

	@Override
	public void initializeRuleBase() {

		// add static rules
		ArrayList<Rule> staticRules = new ArrayList<Rule>();
		staticRules.add(new PermanentSlideDefault(this));
		staticRules.add(new MegaFireballCounter(this));

		// add adaptive rules
		ArrayList<Rule> ruleBase = new ArrayList<Rule>();
		ruleBase.add(new AirSlide(this));
		ruleBase.add(new DistanceAirFireball(this));
		ruleBase.add(new NearAirFireball(this));
		ruleBase.add(new DistanceFireball(this));
		ruleBase.add(new FleeJump(this));
		ruleBase.add(new MegaFireballEnemyDownHighPrio(this));
		ruleBase.add(new MegaFireballTimeLow(this));
		ruleBase.add(new MegaFireballTimeExtremeLow(this));
		ruleBase.add(new FireballCounter(this));
		ruleBase.add(new PermanentHardAirPunch(this));
		ruleBase.add(new PermanentSlideHighPrio(this));
		ruleBase.add(new ShortAttackCounter(this));
		ruleBase.add(new ShortSlideCounter(this));
		ruleBase.add(new SlideCounter(this));
		ruleBase.add(new SlideSlideCombo(this));
		ruleBase.add(new Uppercut(this));
		ruleBase.add(new UppercutAirCounter(this));
		ruleBase.add(new WrongDirectionCounter(this));
		ruleBase.add(new KneeAirCounter(this));
		ruleBase.add(new PermanentSlideCounter(this));
		ruleBase.add(new UppercutAirCounterHighPrio(this));
		ruleBase.add(new KneeAirCounterHighPrio(this));
		ruleBase.add(new PermanentHardAirPunch2(this));
		ruleBase.add(new PermanentHardAirPunch2HighPrio(this));
		ruleBase.add(new PermanentDiagonalFireball(this));
		for (int i = 0; i < MAX_EMPTY_RULE_COUNT; i++) {
			ruleBase.add(new EmptyRule(this));
		}

		rl = new SimpleReinforcementLearning(this);
		rl.setRuleBase(ruleBase.toArray(new Rule[0]));
		rl.setStaticRules(staticRules.toArray(new Rule[0]));

	}

	@Override
	public void initializeDynamicScript() {

		dynamicScript = rl.getDynamicScript(DS_SIZE);

	}

	@Override
	public void updateDynamicScript() {

		rl.update();
		dynamicScript = rl.getDynamicScript(DS_SIZE);

	}

}
