package rl;

import rule.GameStateInterface;
import rule.Rule;

/**
 * Super class for Reinforcement Learning Algorithms that generates scripts from
 * a static rulebase.
 * 
 * @author Kevin Majchrzak
 * 
 */
public abstract class AbstractReinforcementLearning {

	protected Rule[] ruleBase;
	protected Rule[] defaultRules;
	protected GameStateInterface gameState;

	public AbstractReinforcementLearning(GameStateInterface gameState) {
		this.gameState = gameState;
	}

	public void setRuleBase(Rule[] ruleBase) {
		this.ruleBase = ruleBase;
	}

	/**
	 * Generates a new script on the fly.
	 * 
	 * @param length
	 *            the maximum length of the script.
	 * @return the generated script.
	 */
	public abstract Rule[] getDynamicScript(int length);

	/**
	 * Updates the algorithms state.
	 */
	public abstract void update();

	/**
	 * The specified rules will be part of any generated script.
	 * 
	 * @param staticRules
	 *            these rules will be part of any generated script
	 */
	public void setStaticRules(Rule[] staticRules) {
		this.defaultRules = staticRules;
	}

}
