package rule;

import enumerate.State;

/**
 * 
 * @author Kevin Majchrzak
 * 
 */
public class SlideCounter extends Rule {

	public SlideCounter(GameStateInterface gameState) {
		super(gameState);
	}

	@Override
	public boolean satisfied() {

		return (dodgeSatisfied()
				|| (gameState.getMyCharacterData().getSpeedY() > 0) || counterSatisfied())
				&& (ActiveComboRuleWrapper.getSingleton().rule == null);

	}

	private boolean counterSatisfied() {
		return (gameState.getMyCharacterData().getState() == State.AIR)
				&& (gameState.getMyCharacterData().getSpeedY() > -18);
	}

	private boolean dodgeSatisfied() {
		// no need for defense
		if (!gameState.getEnemyCharacterData().getAction().name()
				.equals("STAND_D_DB_BB")) {
			return false;
		}
		// no chance for jump in air
		if (gameState.getMyCharacterData().getState() == State.AIR) {
			return false;
		}
		// no speed left
		if (gameState.getEnemyCharacterData().getRemainingFrame() < 30) {
			return false;
		}

		boolean myCharacterOnLeftSide = gameState.getCommandCenter().getMyX() < gameState
				.getCommandCenter().getEnemyX();

		// enemy is attacking in wrong direction
		if ((gameState.getEnemyCharacterData().isFront() && (myCharacterOnLeftSide))
				|| ((!gameState.getEnemyCharacterData().isFront()) && (!myCharacterOnLeftSide))) {
			return false;
		}

		// check if my character is in enemy attack range
		int enemyAttackNearestHitBoxEdge; // attack hitbox
		int myNearestHitBoxEdge; // character hitbox

		if (myCharacterOnLeftSide) {
			// see Attack.materialise
			enemyAttackNearestHitBoxEdge = gameState.getEnemyCharacterData()
					.getX()
					+ 256
					- gameState.getEnemyMotionData().getAttackHit().getR();
			myNearestHitBoxEdge = gameState.getMyCharacterData().getRight();
		} else {
			enemyAttackNearestHitBoxEdge = gameState.getEnemyCharacterData()
					.getX()
					+ gameState.getEnemyMotionData().getAttackHit().getR();
			myNearestHitBoxEdge = gameState.getMyCharacterData().getLeft();
		}

		// distance between my character and enemy attack hitbox
		int distX = Math
				.abs(enemyAttackNearestHitBoxEdge - myNearestHitBoxEdge);

		// remaining distance enemy will travel with slide
		int absSpeedX = Math.abs(gameState.getEnemyCharacterData().getSpeedX());
		int restDistX = (absSpeedX * (absSpeedX + 1)) / 2; // Gau

		// return true for dodge if my character would be hit
		return distX < restDistX;
	}

	@Override
	public void execute() {

		if (gameState.getMyCharacterData().getState() == State.AIR) {
			if (gameState.getMyCharacterData().getSpeedY() > 0) {
				gameState.getCommandCenter().commandCall("2 1 4 _ B");
			} else {
				execCounter();
			}
		} else {
			execDodge();
		}

	}

	private void execCounter() {
		gameState.getCommandCenter().commandCall("AIR_D_DF_FB");
	}

	private void execDodge() {
		String jumpLeft, jumpRight;
		if (gameState.getMyCharacterData().isFront()) {
			jumpLeft = "BACK_JUMP";
			jumpRight = "FOR_JUMP";
		} else {
			jumpRight = "BACK_JUMP";
			jumpLeft = "FOR_JUMP";
		}

		if (Math.abs(gameState.getMyCharacterData().getX()
				- gameState.getEnemyCharacterData().getX()) > 300) {
			gameState.getCommandCenter().commandCall("JUMP");
		}

		if (gameState.getMyCharacterData().getX() < 300) {
			// jump away from left wall
			gameState.getCommandCenter().commandCall(jumpRight);
		} else if (gameState.getMyCharacterData().getX() > gameState
				.getGameData().getStageXMax() - 300) {
			// jump away from right wall
			gameState.getCommandCenter().commandCall(jumpLeft);
		} else if (gameState.getMyCharacterData().getX() < gameState
				.getEnemyCharacterData().getX()) {
			// jump away from enemy
			gameState.getCommandCenter().commandCall(jumpLeft);
		} else {
			// jump away from enemy
			gameState.getCommandCenter().commandCall(jumpRight);
		}
	}

	@Override
	public int getPriority() {
		return -2;
	}

}
