package FuzzySystem;

import java.util.ArrayList;

public class Rule {
	ArrayList<Antecedent> condition = new ArrayList<Antecedent>();
	ArrayList<Consequent> effect = new ArrayList<Consequent>();
	RuleList effectBehaviour = null;
	
	double ruleWeight;
	
	public Rule()
	{
		ruleWeight = 1;
	}
	
	public double getRuleFiringStrength(LinguisticList varList)
	{
		double firingStrength = 0;
		for(int i=0; i<condition.size(); i++)
		{
			double aFire = condition.get(i).getAFire(varList);
			
			//determine logical operator to be applied to condition's firing strength
			if(i > 0)
			{
				switch(condition.get(i-1).op) //when operator applies to the next condition
				{
				case AND:
				case NOT:
					firingStrength = Math.min(aFire, firingStrength); break;
				case OR:
					firingStrength = Math.max(aFire, firingStrength); break;
				case NAND:
					firingStrength = Math.min(1-aFire, firingStrength); break;
				case NOR:
					firingStrength = Math.max(1-aFire, firingStrength); break;
				default:
					firingStrength = aFire;
				}
			}
			else if(condition.get(i).op == LogicalOperator.NOT) //when operator applies to the previous condition
				firingStrength = 1-aFire;
			else
				firingStrength = aFire;
		}
		
		return firingStrength;
	}
	
	public Rule add(Antecedent ant)
	{
		condition.add(ant);
		return this;
	}
	
	public Rule add(Consequent con)
	{
		effect.add(con);
		return this;
	}
}
